import { BeforeFooterBlock } from "@/components/Blocks/BeforeFooter";
import { BlogCardItemSkeleton } from "@/components/Blogs/BlogCardItem";
import Blogs from "@/components/Blogs/Blogs";
import HeroOther from "@/components/HeroOther";
import { getDefaultMetadata } from "@/utils/metadata";
import { sanitizePageNumber } from "@/utils/sanitize";
import { Metadata } from "next";
import { headers } from "next/headers";
import { Suspense } from "react";

const metaDesc =
  "Explore the latest insights, news, and patient resources on the Cochise Oncology blog. Stay informed about cancer treatments, patient support services, and wellness tips. Read our expert articles today.";

export async function generateMetadata(): Promise<Metadata> {
  const metadata = await getDefaultMetadata();
  metadata.title = `Blog - ${metadata.openGraph?.siteName}`;
  metadata.description = metaDesc;

  return metadata;
}

export default async function BlogPage({ searchParams }: { searchParams?: Promise<{ page?: string; s?: string }> }) {
  const params = await searchParams;
  const paramsPage = params?.page;
  const paramsSearch = params?.s;
  const page = sanitizePageNumber(paramsPage);
  const headersList = await headers();
  const mainUrl = headersList.get("x-main-url");
  const fullUrl = headersList.get("x-full-url");
  const siteName = headersList.get("x-site-name");

  const jsonLd = {
    "@context": "https://schema.org",
    "@graph": [
      {
        "@type": "WebPage",
        "@id": fullUrl,
        url: fullUrl,
        name: `Blog - ${siteName}`,
        isPartOf: {
          "@id": `${mainUrl}/#website`,
        },
        datePublished: "2024-07-18T16:42:14+00:00",
        dateModified: "2025-02-27T01:20:38+00:00",
        description: metaDesc,
        breadcrumb: {
          "@id": `${fullUrl}#breadcrumb`,
        },
        inLanguage: "en-US",
        potentialAction: [
          {
            "@type": "ReadAction",
            target: [fullUrl],
          },
        ],
      },
      {
        "@type": "WebSite",
        "@id": `${mainUrl}/#website`,
        url: `${mainUrl}/`,
        name: siteName,
        description: "",
        publisher: {
          "@id": `${mainUrl}/#organization`,
        },
        potentialAction: [
          {
            "@type": "SearchAction",
            target: {
              "@type": "EntryPoint",
              urlTemplate: `${mainUrl}/?s={search_term_string}`,
            },
            "query-input": {
              "@type": "PropertyValueSpecification",
              valueRequired: true,
              valueName: "search_term_string",
            },
          },
        ],
        inLanguage: "en-US",
      },
      {
        "@type": "Organization",
        "@id": `${mainUrl}/#organization`,
        name: siteName,
        url: `${mainUrl}/`,
        logo: {
          "@type": "ImageObject",
          inLanguage: "en-US",
          "@id": `${mainUrl}/#/schema/logo/image/`,
          url: `${mainUrl}/assets/images/logo-dark.webp`,
          contentUrl: `${mainUrl}/assets/images/logo-dark.webp`,
          caption: siteName,
        },
        image: {
          "@id": `${mainUrl}/#/schema/logo/image/`,
        },
      },
    ],
  };

  return (
    <>
      <script type="application/ld+json" dangerouslySetInnerHTML={{ __html: JSON.stringify(jsonLd) }} />
      <HeroOther title="Blog" />

      <section className="page-section" id="blog">
        <div className="w-full max-w-7xl mx-auto px-4 md:px-8">
          <form action="/blog" method="GET">
            <div className="input-group">
              <input
                type="text"
                name="s"
                defaultValue={paramsSearch ?? ""}
                placeholder="Search blog..."
                className="input-lg input-circle form-control h-12"
              />
              <div className="input-group-append px-2">
                <button className="btn btn-info text-white h-12 px-5" type="submit">
                  Search
                </button>
              </div>
            </div>
          </form>
        </div>

        <Suspense fallback={<Loading />}>
          <Blogs page={page} search={paramsSearch} />
        </Suspense>
      </section>

      <BeforeFooterBlock />
    </>
  );
}

function Loading() {
  return (
    <div className="container position-relative">
      <div className="row">
        <BlogCardItemSkeleton />
        <BlogCardItemSkeleton />
        <BlogCardItemSkeleton />
      </div>
    </div>
  );
}
