import HeroOther from "@/components/HeroOther";
import { fetchBlogDetail } from "@/services/payload/blog";
import { RichText } from "@payloadcms/richtext-lexical/react";
import { headers } from "next/headers";
import Image from "next/image";
import { notFound } from "next/navigation";
import BlogsSuggestion from "./BlogsSuggestion";

export interface BlogDetailProps {
  slug: string | undefined;
}

export default async function BlogDetail({ slug }: BlogDetailProps) {
  const data = await fetchBlogDetail(slug);
  const headersList = await headers();
  const mainUrl = headersList.get("x-main-url");
  const fullUrl = headersList.get("x-full-url");

  const shareUrl = {
    facebook: `https://www.facebook.com/sharer/sharer.php?u=${fullUrl}`,
    linkedin: `https://www.linkedin.com/sharing/share-offsite/?url=${fullUrl}`,
    twitter: `https://twitter.com/intent/tweet?url=${fullUrl}`,
  };

  if (!data) {
    notFound();
  }

  let author = {};
  if (!!data?.data?.createdBy && typeof data?.data?.createdBy !== "number") {
    author = {
      author: {
        name: data?.data?.createdBy?.name,
        "@id": `${mainUrl}/#/schema/person/${data?.data?.createdBy?.id}`,
      },
    };
  }

  const jsonLd = {
    "@context": "https://schema.org",
    "@graph": [
      {
        "@type": "Article",
        "@id": `${fullUrl}#article`,
        isPartOf: {
          "@id": fullUrl,
        },
        ...author,
        headline: data?.data?.title ?? "",
        datePublished: data?.createdAt,
        dateModified: data?.updatedAt,
        mainEntityOfPage: {
          "@id": fullUrl,
        },
        publisher: { "@id": `${mainUrl}/#organization` },
        image: {
          "@id": `${fullUrl}#primaryimage`,
        },
        thumbnailUrl: data?.imgUrl ?? "",
        inLanguage: "en-US",
      },
      {
        "@type": "WebPage",
        "@id": fullUrl,
        url: fullUrl,
        name: `${data?.data?.title ?? ""} - Cochise Oncology`,
        isPartOf: { "@id": `${mainUrl}/#website` },
        primaryImageOfPage: {
          "@id": `${fullUrl}#primaryimage`,
        },
        image: {
          "@id": `${fullUrl}#primaryimage`,
        },
        thumbnailUrl: data?.imgUrl ?? "",
        datePublished: data?.createdAt,
        dateModified: data?.updatedAt,
        description: data?.data?.meta?.description,
        breadcrumb: {
          "@id": `${fullUrl}#breadcrumb`,
        },
        inLanguage: "en-US",
        potentialAction: [
          {
            "@type": "ReadAction",
            target: [fullUrl],
          },
        ],
      },
    ],
  };

  return (
    <>
      <script type="application/ld+json" dangerouslySetInnerHTML={{ __html: JSON.stringify(jsonLd) }} />

      <HeroOther title={data.data.title} createdAt={data?.createdAt} shareUrl={shareUrl} />

      {/* Section */}
      <section className="page-section">
        <div className="container relative">
          <div className="row">
            {/* Content */}
            <div className="col-md-10 offset-md-1 col-lg-10 offset-lg-1">
              {/* Post */}
              <div className="blog-item mb-80 mb-xs-40">
                <div className="blog-item-body">
                  <div className="mb-40 mb-xs-30">
                    <Image src={data.imgUrl} alt="Image Description" width={1350} height={759} className="round" />
                  </div>
                  <div>
                    <RichText data={data.data.content} />
                  </div>
                </div>
              </div>
              {/* End Post */}
            </div>
            {/* End Content */}
            <hr style={{ width: "100%", border: "1px solid #aaa" }} />
          </div>
        </div>
      </section>
      {/* End Section */}
      <div className="mb-5">
        <BlogsSuggestion />
      </div>
    </>
  );
}

export function BlogDetailContentSkeleton() {
  return (
    <section className="page-section animate-pulse">
      <div className="container relative">
        <div className="row">
          {/* Content */}
          <div className="col-md-10 offset-md-1 col-lg-10 offset-lg-1">
            {/* Post */}
            <div className="blog-item mb-80 mb-xs-40">
              <div className="blog-item-body">
                <div className="mb-40 mb-xs-30">
                  <div className="w-full h-72 bg-gray-300 rounded-lg"></div>
                  <div className="h-5 bg-gray-300 rounded w-full mt-3"></div>
                  <div className="h-5 bg-gray-300 rounded w-3/4 mt-3"></div>
                  <div className="h-5 bg-gray-300 rounded w-2/3 mt-3"></div>
                </div>
                <div></div>
              </div>
            </div>
            {/* End Post */}
          </div>
          {/* End Content */}
        </div>
      </div>
    </section>
  );
}
