import { BeforeFooterBlock } from "@/components/Blocks/BeforeFooter";
import { RenderBlocks } from "@/components/Blocks/RenderBlocks";
import BlogDetail from "@/components/Blogs/BlogDetail";
import { fetchPageBySlug } from "@/services/payload/page";
import HeroOther from "@/components/HeroOther";
import { headers } from "next/headers";

export interface PageProps {
  slug: string | undefined;
}

export default async function Page({ slug }: PageProps) {
  const page = await fetchPageBySlug({ slug });

  // if page is a blog post
  if (!page) {
    return (
      <>
        <BlogDetail slug={slug} />;
        <BeforeFooterBlock />
      </>
    );
  }

  // if page is not a blog post
  const headersList = await headers();
  const mainUrl = headersList.get("x-main-url");
  const fullUrl = headersList.get("x-full-url");
  const siteName = headersList.get("x-site-name");
  const metaDesc = !!page?.meta?.description ? page.meta?.description : headersList.get("x-meta-desc");

  const jsonLd = {
    "@context": "https://schema.org",
    "@graph": [
      {
        "@type": "WebPage",
        "@id": fullUrl,
        url: fullUrl,
        name: `${page.meta?.title ?? ""} - ${siteName}`,
        isPartOf: {
          "@id": `${mainUrl}/#website`,
        },
        datePublished: page?.createdAt,
        dateModified: page?.updatedAt,
        description: metaDesc,
        breadcrumb: {
          "@id": `${fullUrl}#breadcrumb`,
        },
        inLanguage: "en-US",
        potentialAction: [
          {
            "@type": "ReadAction",
            target: [fullUrl],
          },
        ],
      },
      {
        "@type": "WebSite",
        "@id": `${mainUrl}/#website`,
        url: `${mainUrl}/`,
        name: siteName,
        description: "",
        publisher: {
          "@id": `${mainUrl}/#organization`,
        },
        potentialAction: [
          {
            "@type": "SearchAction",
            target: {
              "@type": "EntryPoint",
              urlTemplate: `${mainUrl}/?s={search_term_string}`,
            },
            "query-input": {
              "@type": "PropertyValueSpecification",
              valueRequired: true,
              valueName: "search_term_string",
            },
          },
        ],
        inLanguage: "en-US",
      },
      {
        "@type": "Organization",
        "@id": `${mainUrl}/#organization`,
        name: siteName,
        url: `${mainUrl}/`,
        logo: {
          "@type": "ImageObject",
          inLanguage: "en-US",
          "@id": `${mainUrl}/#/schema/logo/image/`,
          url: `${mainUrl}/assets/images/logo-dark.webp`,
          contentUrl: `${mainUrl}/assets/images/logo-dark.webp`,
          width: 500,
          height: 195,
          caption: siteName,
        },
        image: {
          "@id": `${mainUrl}/#/schema/logo/image/`,
        },
      },
    ],
  };

  return (
    <>
      <script type="application/ld+json" dangerouslySetInnerHTML={{ __html: JSON.stringify(jsonLd) }} />
      <HeroOther title={page.title} img={page?.heroImg?.url ?? ""} />
      <RenderBlocks blocks={page.layout} />
    </>
  );
}
